"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Econobit = void 0;
const events_1 = require("events");
const exchange_common_1 = require("exchange-common");
const libs_tools_1 = require("libs-tools");
const makeApiLogin_1 = require("../functions/makeApiLogin");
const onMessage_1 = require("../functions/onMessage");
const onOrderbookUpdate_1 = require("../functions/onOrderbookUpdate");
const onResponse_1 = require("../functions/onResponse");
const openConnection_1 = require("../functions/openConnection");
class Econobit {
    constructor(options = { debug: false, testnet: false }) {
        this.emitter = new events_1.EventEmitter();
        this.isLoggedIn = false;
        this.orderbooks = {};
        this.getWS = () => {
            if (!this.ws || this.ws.readyState !== this.ws.OPEN) {
                throw new Error('you are not connected!');
            }
            return this.ws;
        };
        this.subscribe = {
            ticker: async () => {
                this.send(exchange_common_1.ActionCreators.Subscribe.tickers());
                return {
                    list: (await onResponse_1.onResponse(this.emitter, exchange_common_1.ActionTypes.Ticker.Load)).tickers.list,
                    onUpdate: (callback) => {
                        this.emitter.on(exchange_common_1.ActionTypes.Ticker.Update, (action) => callback(action.ticker));
                    },
                };
            },
            orderbook: async (market) => {
                this.send(exchange_common_1.ActionCreators.Subscribe.orderbook(market));
                const { orderbook: ob, priceDecimals, amountDecimals, symbol } = (await onResponse_1.onResponse(this.emitter, exchange_common_1.ActionTypes.Orderbook.Load));
                this.orderbooks[symbol] = ob;
                const reducer = (prev, curr) => {
                    const price = curr.price.toFixed(priceDecimals);
                    const amount = libs_tools_1.round(curr.amount - curr.filled, amountDecimals);
                    if (!prev[price]) {
                        prev[price] = amount;
                        return prev;
                    }
                    prev[price] = libs_tools_1.round(prev[price] + amount, amountDecimals);
                    return prev;
                };
                const orderbook = {
                    timestamp: new Date([...ob.bids, ...ob.asks].reduce((prev, curr) => Math.max(prev, new Date(curr.date).getTime()), 0)),
                    bids: ob.bids.reduce(reducer, {}),
                    asks: ob.asks.reduce(reducer, {}),
                    symbol,
                };
                return {
                    orderbook,
                    onUpdate: (callback) => {
                        this.emitter.on(`${exchange_common_1.ActionTypes.Orderbook.Update}.${symbol}`, (side, price, amount) => {
                            callback({ side, price: price.toFixed(priceDecimals), amount: libs_tools_1.round(amount, amountDecimals) });
                        });
                    }
                };
            },
            lastTrades: async (market) => {
                this.send(exchange_common_1.ActionCreators.Subscribe.lastTrades(market));
                const { lastTrades } = await onResponse_1.onResponse(this.emitter, exchange_common_1.ActionTypes.LastTrades.Load);
                return {
                    list: lastTrades.list[market.toUpperCase()],
                    onAdd: (callback) => {
                        this.emitter.on(exchange_common_1.ActionTypes.LastTrades.Add, (action) => callback(action.trade));
                    }
                };
            },
            balances: async () => {
                this.send(exchange_common_1.ActionCreators.Subscribe.balances());
                return {
                    list: (await onResponse_1.onResponse(this.emitter, exchange_common_1.ActionTypes.Wallet.List, exchange_common_1.ActionTypes.Message.Errors)).wallets.list,
                    onUpdate: (callback) => {
                        this.emitter.on(exchange_common_1.ActionTypes.Wallet.Update, (action) => callback(action.wallet));
                    },
                };
            },
            openOrders: async () => {
                this.send(exchange_common_1.ActionCreators.Subscribe.openOrders());
                return {
                    list: (await onResponse_1.onResponse(this.emitter, exchange_common_1.ActionTypes.OpenOrders.Load, exchange_common_1.ActionTypes.Message.Errors)).list,
                    onAdd: (callback) => {
                        this.emitter.on(exchange_common_1.ActionTypes.OpenOrders.Add, (action) => callback(action.order));
                    },
                    onUpdate: (callback) => {
                        this.emitter.on(exchange_common_1.ActionTypes.OpenOrders.Update, (action) => callback(action.order));
                    },
                    onDelete: (callback) => {
                        this.emitter.on(exchange_common_1.ActionTypes.OpenOrders.Delete, (action) => callback(action.orderId));
                    },
                };
            },
            ordersHistory: async () => {
                this.send(exchange_common_1.ActionCreators.Subscribe.ordersHistory());
                return {
                    list: (await onResponse_1.onResponse(this.emitter, exchange_common_1.ActionTypes.OrdersHistory.Load, exchange_common_1.ActionTypes.Message.Errors)).list,
                    onAdd: (callback) => {
                        this.emitter.on(exchange_common_1.ActionTypes.OrdersHistory.Add, (action) => callback(action.order));
                    },
                };
            },
            deposits: async () => {
                this.send(exchange_common_1.ActionCreators.Subscribe.deposits());
                return {
                    list: (await onResponse_1.onResponse(this.emitter, exchange_common_1.ActionTypes.Deposit.List, exchange_common_1.ActionTypes.Message.Errors)).deposits.list,
                    onAdd: (callback) => {
                        this.emitter.on(exchange_common_1.ActionTypes.Deposit.Add, (action) => callback(action.deposit));
                    },
                    onUpdate: (callback) => {
                        this.emitter.on(exchange_common_1.ActionTypes.Deposit.Update, (action) => callback(action.deposit));
                    },
                };
            },
            withdrawals: async () => {
                this.send(exchange_common_1.ActionCreators.Subscribe.withdrawals());
                return {
                    list: (await onResponse_1.onResponse(this.emitter, exchange_common_1.ActionTypes.Withdraw.List, exchange_common_1.ActionTypes.Message.Errors)).withdrawals.list,
                    onAdd: (callback) => {
                        this.emitter.on(exchange_common_1.ActionTypes.Withdraw.Add, (action) => callback(action.withdraw));
                    },
                    onUpdate: (callback) => {
                        this.emitter.on(exchange_common_1.ActionTypes.Withdraw.Update, (action) => callback(action.withdraw));
                    },
                };
            },
            trades: async () => {
                this.send(exchange_common_1.ActionCreators.Subscribe.trades());
                return {
                    list: (await onResponse_1.onResponse(this.emitter, exchange_common_1.ActionTypes.Trades.List, exchange_common_1.ActionTypes.Message.Errors)).trades,
                    onAdd: (callback) => {
                        this.emitter.on(exchange_common_1.ActionTypes.Trades.Add, (action) => callback(action.trade));
                    },
                };
            },
        };
        this.unsubscribe = {
            ticker: () => {
                this.send(exchange_common_1.ActionCreators.Unsubscribe.tickers());
                this.emitter.removeAllListeners(exchange_common_1.ActionTypes.Ticker.Update);
            },
            orderbook: (symbol) => {
                this.send(exchange_common_1.ActionCreators.Unsubscribe.orderbook(symbol));
                this.emitter.removeAllListeners(`${exchange_common_1.ActionTypes.Orderbook.Update}.${symbol.toUpperCase()}`);
            },
            lastTrades: (symbol) => {
                this.send(exchange_common_1.ActionCreators.Unsubscribe.lastTrades(symbol));
                this.emitter.removeAllListeners(exchange_common_1.ActionTypes.LastTrades.Add);
            },
            balances: async () => {
                this.send(exchange_common_1.ActionCreators.Unsubscribe.balances());
                this.emitter.removeAllListeners(exchange_common_1.ActionTypes.Wallet.Update);
                return onResponse_1.onResponse(this.emitter, exchange_common_1.ActionTypes.Wallet.Unload, exchange_common_1.ActionTypes.Message.Errors);
            },
            openOrders: () => {
                this.send(exchange_common_1.ActionCreators.Unsubscribe.openOrders());
                this.emitter.removeAllListeners(exchange_common_1.ActionTypes.OpenOrders.Add);
                this.emitter.removeAllListeners(exchange_common_1.ActionTypes.OpenOrders.Update);
                this.emitter.removeAllListeners(exchange_common_1.ActionTypes.OpenOrders.Delete);
            },
            ordersHistory: () => {
                this.send(exchange_common_1.ActionCreators.Unsubscribe.ordersHistory());
                this.emitter.removeAllListeners(exchange_common_1.ActionTypes.OrdersHistory.Add);
            },
            deposits: () => {
                this.send(exchange_common_1.ActionCreators.Unsubscribe.deposits());
                this.emitter.removeAllListeners(exchange_common_1.ActionTypes.Deposit.Add);
                this.emitter.removeAllListeners(exchange_common_1.ActionTypes.Deposit.Update);
            },
            withdrawals: () => {
                this.send(exchange_common_1.ActionCreators.Unsubscribe.withdrawals());
                this.emitter.removeAllListeners(exchange_common_1.ActionTypes.Withdraw.Add);
                this.emitter.removeAllListeners(exchange_common_1.ActionTypes.Withdraw.Update);
            },
            trades: () => {
                this.send(exchange_common_1.ActionCreators.Unsubscribe.trades());
                this.emitter.removeAllListeners(exchange_common_1.ActionTypes.Trades.Add);
            },
        };
        this.order = {
            send: async (order) => {
                this.send(exchange_common_1.ActionCreators.Order.insert(order));
                return await onResponse_1.onResponse(this.emitter, exchange_common_1.ActionTypes.Message.Success, exchange_common_1.ActionTypes.Message.Errors);
            },
            cancel: async (order) => {
                this.send(exchange_common_1.ActionCreators.Order.delete(order));
                return await onResponse_1.onResponse(this.emitter, exchange_common_1.ActionTypes.Message.Success, exchange_common_1.ActionTypes.Message.Errors);
            }
        };
        this.bulkOrders = {
            send: async (orders) => {
                this.send(exchange_common_1.ActionCreators.BulkOrders.insert(orders));
                return await onResponse_1.onResponse(this.emitter, exchange_common_1.ActionTypes.Message.Success, exchange_common_1.ActionTypes.Message.Errors);
            },
            cancel: async (cancelBulkOrders) => {
                this.send(exchange_common_1.ActionCreators.BulkOrders.delete(cancelBulkOrders));
                return await onResponse_1.onResponse(this.emitter, exchange_common_1.ActionTypes.Message.Success, exchange_common_1.ActionTypes.Message.Errors);
            }
        };
        this.debug = options.debug || false;
        this.testnet = options.testnet || false;
        onOrderbookUpdate_1.onOrderbookUpdate(this.emitter, this.orderbooks);
    }
    send(action) {
        const ws = this.getWS();
        ws.send(JSON.stringify(action));
    }
    connect() {
        const ws = this.ws = openConnection_1.openConnection(this.testnet);
        ws.on('error', libs_tools_1.error);
        ws.on('message', (msg) => {
            onMessage_1.onMessage(this.emitter)(msg);
            this.debug && libs_tools_1.info(JSON.parse(msg));
        });
        ws.on('close', () => {
            ws.removeAllListeners();
            this.emitter.removeAllListeners();
        });
        return new Promise(resolve => {
            ws.once('open', () => resolve(ws));
        });
    }
    disconnect() {
        const ws = this.getWS();
        ws.terminate();
    }
    get isLogged() {
        return this.isLoggedIn;
    }
    get isConnected() {
        return this.ws && this.ws.readyState === this.ws.OPEN || false;
    }
    async login(apiSign) {
        const apiLogin = makeApiLogin_1.makeApiLogin(apiSign);
        this.send(exchange_common_1.ActionCreators.ApiKey.login(apiLogin));
        await onResponse_1.onResponse(this.emitter, exchange_common_1.ActionTypes.ApiKey.Success, exchange_common_1.ActionTypes.ApiKey.Fail);
        this.isLoggedIn = true;
    }
    async logout() {
        this.send(exchange_common_1.ActionCreators.ApiKey.logout());
        await onResponse_1.onResponse(this.emitter, exchange_common_1.ActionTypes.ApiKey.Success, exchange_common_1.ActionTypes.ApiKey.Fail);
        this.isLoggedIn = false;
    }
}
exports.Econobit = Econobit;
//# sourceMappingURL=econobit.js.map